// ============================================================================
// $Id: intermud.cc,v 1.32 1997/10/22 14:39:25 chris Exp $
//
// This is part of Realm Of Magic
//
// Usage: This is the intermud client to receive and send udp packages
//
// All rights reserved. See license.doc for complete information.
//
// Copyright (C) 1993-1996 by the Hackerz of the Realm Of Magic
// Realm Of Magic is based on CircleMUD, Copyright (C) 1993.
// CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.
// Original intermud_client.c (C) 1995, 96 by Chris Austin (Stryker@Tempus)
// Original lpc protocol (C) by Mark Lewis (Nostradamus@Zebedee)
//                           by J.W. Armstrong (Alvin@Sushi)
//
// ============================================================================

#include <string>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <list>

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>

#include "imuddefs.hh"
#include "log.hh"
#include "net.hh"
#include "mud.hh"
#include "utils.hh"
#include "tokenizer.hh"
#include "circle_protocol.hh"
#include "lpc_protocol.hh"
#include "protocol.hh"

// The dump table contains the ip adresses that tried to reach us
// but were not in one of the mud lists

list<unsigned long> dump_table;


int lpc_id_number;
lpc_info_record lpc_data;
_circle circlemuds;
_lpc lpcmuds;
_log interlog;
_mud localmud;
_net network;

/* Local function prototypes */

void   server_loop();
void   recv_net_message();
void   recv_mud_message();
void   purge_dns_table();
void   purge_dump_table();
void   startup();
void   mud_recv_mudlist_req(string);
void   mud_recv_intertell(string, string, string, string);
void   mud_recv_intertell_nr(string, string, string, string);
void   mud_recv_interwho_req(string, string);
void   mud_recv_interwho_rpl(string, string, string);
void   mud_recv_interpage(string, string, string, string);
void   mud_recv_interwiz(string, string);
void   mud_recv_mudinfo(string, string);
void   mud_recv_dnspurge(string);
void   mud_recv_reboot(string);
void   mud_recv_stats(string);
void   mud_recv_mute(string, string);
void   mud_recv_debug();
void   mud_recv_shutdown(string);
void   net_keep_alive();
void read_dump_table();
bool access_control(struct sockaddr_in, string &);
void close_sockets();

int main (int argc, char **argv)
{
  int port;
  
  if (argc != 2)  {
    cerr << "Usage: " << argv[0] << " [net port #]" << endl;
    exit(1);
  }
  
  if (!isdigit(*argv[1]))  {
    cerr << "Usage: " << argv[0] << " [net port #]" << endl;
    exit(1);
  }
  else if ((port = atoi(argv[1])) <= 1024)  {
    cerr << "Illegal port number specified." << endl;
    exit(1);
  }
 
  lpc_id_number = 0;
  
  network.Init(port);
  
  startup();
  
  server_loop();
 
  close_sockets();
  
  exit(0);
}


void startup()
{
 
  circlemuds.Startup();
  lpcmuds.Startup(); 
  interlog << "INFO: Mudlist read ok!" << logit;
 
  // Reading those muds, who wanted access but were denied.
  
  if (dump_table.empty())
    read_dump_table();
  interlog << "INFO: Dump Table read ok!" << logit;
  
  return;
}


void server_loop()
{
  struct timeval timeout;
  fd_set input_set, except_set;
  int    retval;
  
  
  timeout.tv_sec = 300;
  timeout.tv_usec = 0;
     
  for (;;) {
   
    localmud.AcceptConnection();

    do {
    
      if (lpc_id_number > 65536)
        lpc_id_number = 0;
    
      FD_ZERO(&input_set);
      FD_ZERO(&except_set);
      network.AddToSet(input_set);
      localmud.AddToSet(input_set);
      localmud.AddToSet(except_set);
      network.AddToSet(except_set);
    
      errno = 0;
   
      retval = select(localmud.Desc() + 1, &input_set, (fd_set *) 0, 
                      &except_set, &timeout);
      
      if (retval < 0)
        interlog << "INFO: Waking up to process signal" << logit;
      else  {
        if (retval == 0)  {
          net_keep_alive();
	  timeout.tv_sec = 300;
        }
        if (network.CheckSet(except_set))
          interlog << "SYSERR: Exception detected on net socket!" << logit;
        if (localmud.CheckSet(except_set))  {
          interlog << "SYSERR: Exception detected on mud socket!" << logit;
        
          localmud.ClearSet(input_set);
          localmud.ClearSet(except_set);
          close_sockets();
          exit(1);
        }
        
        if (network.CheckSet(input_set))
          recv_net_message();
        if (localmud.CheckSet(input_set))
          recv_mud_message();
      }
    } while (!localmud.MudLost());
    localmud.CloseFD();
  }
  return;
}

void net_keep_alive()  
{
  circlemuds.SendPings();
  lpcmuds.SendPings(); 
  return;
}

void recv_net_message()
{
  struct sockaddr_in remote_address;
  string message;
  
  message = network.ReceiveData(remote_address); 
  if (!message.empty()) {
    
    if (!access_control(remote_address, message)) 
      return;
  
    if (circlemuds.IncomingMessage(message, remote_address))
      return;
   
    if (lpcmuds.IncomingMessage(message, remote_address))
      return;
  
    interlog << "SYSERR: Unknown message received from "
             << inet_ntoa(remote_address.sin_addr) << ")" << logit;
    interlog << "Message was: " << message << logit;
  }
  return;
}

bool access_control(struct sockaddr_in remote_address, string &mess)
{
  ofstream add_dump(ADDRESS_DUMP, ios::out | ios::app);

  if (circlemuds.VerifyAccess(remote_address, mess))
    return true;

  if (lpcmuds.VerifyAccess(remote_address, mess))
    return true;
  
  list<unsigned long>::iterator lipos2;
  for (lipos2 = dump_table.begin();lipos2 != dump_table.end(); ++lipos2)
    if (*lipos2 == remote_address.sin_addr.s_addr) 
      return false;
  
  dump_table.push_back(remote_address.sin_addr.s_addr);
  add_dump << inet_ntoa(remote_address.sin_addr) << endl;
  add_dump.close();
 
  interlog << "INFO: Denied access from " << inet_ntoa(remote_address.sin_addr)
           << ", message: " << mess << logit;

  localmud << "mudlog#@#INFO: Intermud denied access from "
           << inet_ntoa(remote_address.sin_addr) << sendit;
  
  network.SetAddr(remote_address.sin_addr.s_addr);
  network.SetPort(remote_address.sin_port);
  if (isdigit(mess[0])) {
    network << "3000|Server|" << MUDNAME << "|You are not in the access list "
               " of " << MUDNAME << ". If you want access, please send a mail "
               "to: " << INTERMUD_EMAIL << "|";
  } else {
    network << "Channel:intercode|REQ:channel|SND:Server|NAME:"
            << MUDNAME << "|UDP:" << INTERMUD_PORT << "|DATA:"
               "You are not in the access list of " << MUDNAME
            << ". If you want access, please send a mail to: "
            << INTERMUD_EMAIL;
  }
  network << sendit;
  return false;
}

void read_dump_table()
{
  ifstream dumpfile(ADDRESS_DUMP);
  char dummychar[8192];

  if (!dumpfile) {
    interlog << "SYSERR: No dumpfile - exiting" << logit;
    close_sockets();
    exit(1);
  }

  while(!dumpfile.eof()) {
    dumpfile.getline(dummychar, 8192, '\n');
    if (isdigit(dummychar[0])) 
      dump_table.push_back(inet_addr(dummychar));
  }
  dumpfile.close();
  return;
}

void recv_mud_message(void)
{
  string message;
  
  message = localmud.ReceiveData();
  
  if (!message.empty()) {
    tokenizer message_tokens(message, "#@#");
    
    if (!case_compare(message_tokens[0], "mudinfo"))
      mud_recv_mudinfo(message_tokens[1], message_tokens[2]);
    else if (!case_compare(message_tokens[0], "mudlist"))
      mud_recv_mudlist_req(message_tokens[1]);
    else if (!case_compare(message_tokens[0], "interpage"))
      mud_recv_interpage(message_tokens[1], message_tokens[2],
                         message_tokens[3], message_tokens[4]);
    else if (!case_compare(message_tokens[0], "who-query"))
      mud_recv_interwho_req(message_tokens[1], message_tokens[2]);
    else if (!case_compare(message_tokens[0], "who-reply"))
      mud_recv_interwho_rpl(message_tokens[1], message_tokens[2],
                            message_tokens[3]);
    else if (!case_compare(message_tokens[0], "intertell"))
      mud_recv_intertell(message_tokens[1], message_tokens[2],
                         message_tokens[3], message_tokens[4]);
    else if (!case_compare(message_tokens[0], "intertell-nr"))
      mud_recv_intertell_nr(message_tokens[1], message_tokens[2],
                            message_tokens[3], message_tokens[4]);
    else if (!case_compare(message_tokens[0], "interwiz"))
      mud_recv_interwiz(message_tokens[1], message_tokens[2]);
    else if (!case_compare(message_tokens[0], "purge-mudlist"))
      mud_recv_dnspurge(message_tokens[1]);
    else if (!case_compare(message_tokens[0], "reboot-mudlist"))
      mud_recv_reboot(message_tokens[1]);
    else if (!case_compare(message_tokens[0], "intermud-stats"))
      mud_recv_stats(message_tokens[1]);
    else if (!case_compare(message_tokens[0], "debug"))
      mud_recv_debug();
    else if (!case_compare(message_tokens[0], "mute"))
      mud_recv_mute(message_tokens[1], message_tokens[2]);
    else if (!case_compare(message_tokens[0], "shutdown"))
      mud_recv_shutdown(message_tokens[1]);
    else 
      interlog << "INFO: Received unknown message-header "
               << message_tokens[0] << " from the MUD." << logit;
  }     
  return;
}

void mud_recv_mudlist_req(string player)
{
  localmud << "inc-mudlist#@#" << player << "#@#";
  
  localmud << "  _____________________________________"
              "_____________________________________\n"
              " /                                     "
              "                                     \\\n"
              "| Name                IP              Port  MUD-Type"
              "  Last contact at        |\n"
              "|---------------------------------------------------"
              "-------------------------|\n";
 
  circlemuds.ListMuds();
  lpcmuds.ListMuds();
  
  localmud << "|____________________________________________"
              "________________________________|\n" << sendit;
  return;
}


void mud_recv_intertell_nr(string recipent, string remote_mud, string player,
                           string data)
{
  if (circlemuds.OutgoingIntertellNR(recipent, remote_mud, player, data))
    return;
 
  if (lpcmuds.OutgoingIntertellNR(recipent, remote_mud, player, data))
    return;

  localmud << "inc-tell-nr#@#Server#@#" << MUDNAME << "#@#" << player
           << "#@#" << remote_mud << " is not in my list, try <mudlist>" 
           << sendit;
  return;
}

void mud_recv_intertell(string recipent, string remote_mud, string player,
                        string data)
{
  if (circlemuds.OutgoingIntertell(recipent, remote_mud, player, data))
    return;
 
  if (lpcmuds.OutgoingIntertell(recipent, remote_mud, player, data))
    return;

  localmud << "inc-tell-nr#@#Server#@#" << MUDNAME << "#@#" << player
           << "#@#" << remote_mud << " is not in my list, try <mudlist>" 
           << sendit;
  return;
}

void mud_recv_interwho_rpl(string recipent, string remote_mud, string data)
{
  if (circlemuds.OutgoingInterwho(recipent, remote_mud, data))
    return;
 
  if (lpcmuds.OutgoingInterwho(recipent, remote_mud, data))
    return;

  interlog << "SYSERR: Out going who did not find fitting reply-packet!"
           << logit;
  localmud << "mudlog#@#INTERMUD SYSERR: Outgoing Interwho did not find "
              "fitting reply-packet!" << sendit;
  return;
}      
    
void mud_recv_interwho_req(string remote_mud, string player)
{
  if (circlemuds.OutgoingInterwhoREQ(remote_mud, player))
    return;

  if (lpcmuds.OutgoingInterwhoREQ(remote_mud, player))
    return;

  localmud << "inc-tell-nr#@#Server#@#" << MUDNAME << "#@#" << player
           << "#@#" << remote_mud << " is not in my list, try <mudlist>" 
           << sendit;
  return;
}
    
    
void mud_recv_interpage(string recipent, string remote_mud, string player,
                        string data)
{
  
  if (circlemuds.OutgoingInterpage(recipent, remote_mud, player, data))
    return;
  
  localmud << "inc-tell-nr#@#Server#@#" << MUDNAME << "#@#" << player
           << "#@#" << remote_mud << " is not in my list, try <mudlist>" 
           << sendit;
  return;
}

    
void mud_recv_interwiz(string player, string data)
{ 
  circlemuds.OutgoingInterwiz(player, data);
  lpcmuds.OutgoingInterwiz(player, data);
  
  return;
}
    
    
void mud_recv_mudinfo(string remote_mud, string player)
{
  if (circlemuds.GetMudinfo(remote_mud, player))
    return;
 
  if (lpcmuds.GetMudinfo(remote_mud, player))
    return;

  
  localmud << "inc-tell-nr#@#Server#@#" << MUDNAME << "#@#" << player
           << "#@#" << remote_mud << " is not in my list, try <mudlist>" 
           << sendit;
  return;
}

    
void mud_recv_dnspurge(string player)
{
  interlog << "(GC) " << player << " requested DNS purge" 
           << logit;
  localmud << "mudlog#@#(GC) INTERMUD: DNS purge by " << player << sendit;
  purge_dns_table();
  return;
}
    
void mud_recv_reboot(string player)
{
  interlog << "(GC) " << player << " rebooted the mudlists" 
           << logit;
  localmud << "mudlog#@#(GC) INTERMUD: Mudlist reboot by " 
           << player << sendit;
  purge_dns_table();
  circlemuds.Startup();
  lpcmuds.Startup();
  return;
}

void mud_recv_stats(string player)
{
  localmud << "inc-stats#@#" << player 
           << "#@#"
           << " ___________________________________________________________"
              "_______________\n"
              "/                                                           "
              "               \\\n"
              "|                      Intermud Network Statistics          "
              "               |\n"
              "|-----------------------------------------------------------"
              "---------------|\n"
              "| Name             | KBytes in | Kbytes out | Mess in | Mess"
              " out |  Muted  |\n"
              "|-----------------------------------------------------------"
              "---------------|\n";

  circlemuds.GetStatistics();
  lpcmuds.GetStatistics();

  localmud << "|__________________________________________________________"
              "________________|\n" << sendit;
  return;
}
    

void mud_recv_mute(string remote_mud, string player)
{
  if (circlemuds.MuteMud(remote_mud, player))
    return;
 
  if (lpcmuds.MuteMud(remote_mud, player))
    return;

  localmud << "inc-tell-nr#@#Server#@#" << MUDNAME << "#@#" << player
           << "#@#" << remote_mud << " is not in my list, try <mudlist>" 
           << sendit;
  return;
}
    
void mud_recv_debug()
{
 
  interlog << "[ DEBUG ] Activating debug code" << logit;

  return;
}

void mud_recv_shutdown(string player)
{
  interlog << "(GC) Shutdown by " << player
           << logit;
  close_sockets();
  exit(0);

}

void purge_dns_table()
{
  circlemuds.PurgeList();
  lpcmuds.PurgeList();
  return;
}

void purge_dump_table()
{
  dump_table.erase(dump_table.begin(), dump_table.end());
  return;
}

void close_sockets()
{
  localmud.CloseFD();
  network.CloseFD();
  return;
}

