// ============================================================================
// $Id: protocol.cc,v 1.1 1996/04/22 18:25:24 questor Exp $
//
// This is part of Realm Of Magic
//
// Usage: Class that serves as the base class for intermud protocols
//
// All rights reserved. See license.doc for complete information.
//
// Copyright (C) 1993-1996 by the Hackerz of the Realm Of Magic
// Realm Of Magic is based on CircleMUD, Copyright (C) 1993.
// CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.
// Original intermud_client.c (C) 1995, 96 by Chris Austin (Stryker@Tempus)
// Original lpc protocol (C) by Mark Lewis (Nostradamus@Zebedee)
//                           by J.W. Armstrong (Alvin@Sushi)
// ============================================================================

#include <string>
#include <list>
#include <arpa/inet.h>
#include <stdio.h>

#include "protocol.hh"
#include "utils.hh"
#include "mud.hh"
#include "log.hh"

extern _mud localmud;
extern _log interlog;

void _protocol::update_statistics(string &remote_name, int direction, 
                                int msg_size)
{
  list<dns_record>::iterator position;
  for (position = dns_table.begin(); position != dns_table.end();
      ++position)
    if (!case_compare(remote_name, (*position).mud_name))
      break;

  if (direction == 0) {
    (*position).kb_out += msg_size;
    (*position).msg_out++;
  } else {
    (*position).kb_in += msg_size;
    (*position).msg_in++;
    update_timestamp(*position);
  }
  return;
}

void _protocol::GetStatistics()
{
  char temp[8192];
  list<dns_record>::iterator position;
  for (position = dns_table.begin(); position != dns_table.end();
      ++position) {
    localmud << "| ";
    if ((*position).mud_name.size() < 17) {
      localmud << (*position).mud_name;
      for (int i = 1; i < (17 - (*position).mud_name.size()); i++)
        localmud << " ";
    } else
      localmud << (*position).mud_name.substr(0, 17);
    localmud << "  ";
    sprintf(temp, "    %d", (*position).kb_in);
    if (strlen(temp) < 11) {
      localmud << temp;
      for (int i = 1; i < (11 - strlen(temp)); i++)
        localmud << " ";
    } else {
      sprintf(temp, "%.11s", temp);
      localmud << temp;
    }
    localmud << "  ";
    sprintf(temp, "    %d", (*position).kb_out);
    if (strlen(temp) < 12) {
      localmud << temp;
      for (int i = 1; i < (12 - strlen(temp)); i++)
        localmud << " ";
    } else {
      sprintf(temp, "%.12s", temp);
      localmud << temp;
    }
    localmud << "  ";
    sprintf(temp, "    %d", (*position).msg_in);
    if (strlen(temp) < 9) {
      localmud << temp;
      for (int i = 1; i < (9 - strlen(temp)); i++)
        localmud << " ";
    } else {
      sprintf(temp, "%.9", temp);
      localmud << temp;
    }
    localmud << "  ";
    sprintf(temp, "    %d", (*position).msg_out);
    if (strlen(temp) < 10) {
      localmud << temp;
      for (int i = 1; i < (10 - strlen(temp)); i++)
        localmud << " ";
    } else {
      sprintf(temp, "%.10s", temp);
      localmud << temp;
    }
    localmud << "  ";
    if ((*position).muted == 1)
      localmud << "   yes   ";
    else
      localmud << "   nop   ";
    localmud << "|\n";
  }
  return;
}

bool _protocol::VerifyAccess(sockaddr_in &remote, string &mess)
{
  list<dns_record>::iterator position;
  for (position = dns_table.begin(); position != dns_table.end();
      ++position)
    if ((*position).ip_address == remote.sin_addr.s_addr) {
      update_statistics((*position).mud_name, 1, mess.size());
      return true;
    }
  return false;
}

void _protocol::ListMuds()
{
  struct in_addr address;
  string tmp_addr;

  list<dns_record>::iterator position;
  for (position = dns_table.begin(); position != dns_table.end();
      ++position) {
    localmud << "| ";
    if ((*position).mud_name.size() < 18) {
      localmud << (*position).mud_name;
      for (int i = (*position).mud_name.size(); i < 18; i++)
        localmud << " ";
    } else
      localmud << (*position).mud_name.substr(0, 18);
    localmud << "  ";
    address.s_addr = (*position).ip_address;
    tmp_addr = inet_ntoa(address);
    if (tmp_addr.size() < 15) {
      localmud << tmp_addr;
      for (int i = tmp_addr.size(); i < 15; i++)
        localmud << " ";
    } else
      localmud << tmp_addr; // not possible to be greater than 15
    localmud << " ";
    if ((*position).mud_port == "23")
      localmud << "----";
    else
      localmud << (*position).mud_port;
    localmud << "  ";
    if (TypeName().size() < 10) {
      localmud << TypeName();
      for (int i = TypeName().size(); i < 10; i++)
        localmud << " ";
    } else
      localmud << TypeName().substr(0, 10);
    if ((*position).last_connected.size() < 19) {
      localmud << (*position).last_connected;
      for (int i = (*position).last_connected.size(); i < 19; i++)
        localmud << " ";
    } else
      localmud << (*position).last_connected.substr(0, 19);
    localmud << "    |\n";
  }
  return;
}

bool _protocol::GetMudinfo(string &remote_mud, string &player)
{
  bool found = false;

  list<dns_record>::iterator position;
  for (position = dns_table.begin(); position != dns_table.end();
      ++position)
    if (!case_compare((*position).mud_name, remote_mud)) {
      found = true;
      break;
    }
  if (!found)
    return false;
  localmud << "inc-mudinfo#@#" << player << "#@#";
  localmud << " _________________________________"
              "________________________________\n";
  localmud << "/  \\                             "
              "                                 \\\n";
  localmud << "|/\\|                              "
              "                                |\n";
  localmud << "\\_/|                               "
              "                               |\n";
  string headline = "Ye Scroll of the Realm of ";
  headline += (*position).mud_name;
  int centerpos = 31 - (headline.size() / 2);
  localmud << "   |";
  for (int i = 1; i <= centerpos; i++)
    localmud << " ";
  localmud << headline;
  if (headline.size() % 2 != 0)
    centerpos--;
  for (int i = 1; i <= centerpos; i++)
    localmud << " ";
  localmud << "|\n";
  localmud << "   |                                       "
              "                       |\n";
  localmud << BasedOn();
  if (IS_SET((*position).services, ST_INTERWIZ))
    localmud << "   |     - Our gods can commune with the gods of this MUD"
                  "         |\n";
  if (IS_SET((*position).services, ST_INTERTELL))
    localmud << "   |     - You can talk to inhabitants of this MUD"
                "                |\n";
  if (IS_SET((*position).services, ST_INTERPAGE))
    localmud << "   |     - Our gods can alarm the gods of this MUD"
                "                |\n";
  if (IS_SET((*position).services, ST_INTERWHO))
    localmud << "   |     - You can see who roams this MUD"
                "                         |\n";
  localmud << "   | ______________________________________________________________\n";
  localmud << "   |/ \\                                                             \\\n";
  localmud << "   |\\/|                                                             |\n";
  localmud << "   \\__/_____________________________________________________________/\n" << sendit;
  return true;
}

bool _protocol::MuteMud(string &remote_mud, string &player)
{
  list<dns_record>::iterator position;
  for (position = dns_table.begin(); position != dns_table.end();
      ++position)
    if (!case_compare((*position).mud_name, remote_mud)) {
      if ((*position).muted == 1) {
        (*position).muted = 0;
        interlog << "(GC) Mute OFF for " << (*position).mud_name 
                 << " by " << player << logit;
        localmud << "mudlog#@#(GC) INTERMUD: Mute OFF for " 
                 << (*position).mud_name << " by " << player << sendit;
      } else {
        (*position).muted = 1;
        interlog << "(GC) Mute ON for " << (*position).mud_name 
                 << " by " << player << logit;
        localmud << "mudlog#@#(GC) INTERMUD: Mute ON for "
                 << (*position).mud_name << " by " << player << sendit;
      }
      return true;
    }
  return false;
}

void _protocol::PurgeList()
{
  dns_table.erase(dns_table.begin(), dns_table.end());
  return;
}


